import orjson
from httpx import AsyncClient

from aiommost import errors
from aiommost.teams.schemas import Team, TeamTypes


class TeamClient:

    def __init__(self, session: AsyncClient) -> None:
        self.session = session

    async def create(self, name: str, display_name: str, team_type: TeamTypes) -> Team:
        """Create public or private team."""
        url = "/teams"
        request = {
            "name": name,
            "display_name": display_name,
            "type": team_type.value,
        }

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        team = orjson.loads(response.content)
        return Team(**team)



import pytest

from aiommost.teams.schemas import TeamTypes


@pytest.fixture
def create_team(faker, client):
    async def inner(
        name: str | None = None,
        display_name: str | None = None,
        team_type: TeamTypes = TeamTypes.PUBLIC,
    ):
        return await client.teams.create(
            name=name or faker.pystr(min_chars=10, max_chars=20).lower(),
            display_name=display_name or faker.pystr(),
            team_type=team_type,
        )

    return inner



import random

import pytest

pytest_plugins = (
    "tests.fixtures.client",
    "tests.fixtures.fake",
    "tests.fixtures.channels",
    "tests.fixtures.teams",
    "tests.fixtures.users",
)


def pytest_make_parametrize_id(config, val):
    return repr(val)


@pytest.fixture
def not_existing_id(faker):
    return faker.uuid4()


@pytest.fixture(scope="session", autouse=True)
def faker_seed():
    return random.random()  # noqa: S311



import orjson
from httpx import AsyncClient

from aiommost import errors
from aiommost.channels.schemas import Channel, ChannelTypes


class ChannelsClient:

    def __init__(self, session: AsyncClient) -> None:
        self.session = session

    async def create_direct(self, first: str, second: str) -> Channel:
        """Create direct channel between users."""
        url = "/channels/direct"
        request = [first, second]

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        channel = orjson.loads(response.content)
        return Channel(**channel)

    async def create(self,
                     team_id: str,
                     name: str,
                     display_name: str,
                     channel_type: ChannelTypes,
                     purpose: str | None = None,
                     header: str | None = None) -> Channel:
        """Create public or private channel."""
        url = "/channels"
        request = {
            "team_id": team_id,
            "name": name,
            "display_name": display_name,
            "purpose": purpose,
            "header": header,
            "type": channel_type.value,
        }

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        channel = orjson.loads(response.content)
        return Channel(**channel)

    async def get_by_id(self, channel_id: str) -> Channel:
        """Get channel info by channel id."""
        url = f"/channels/{channel_id}"

        response = await self.session.get(url)
        errors.validate(response)

        channel = orjson.loads(response.content)
        return Channel(**channel)



import httpx

from aiommost.channels.client import ChannelsClient
from aiommost.posts.client import PostClient
from aiommost.teams.client import TeamClient
from aiommost.users.client import UsersClient


class MattermostClient:

    def __init__(self, host: str, token: str, verify: bool | str = True) -> None:
        self.token = token
        self.verify = verify

        self.session = httpx.AsyncClient(
            base_url=f"{host}/api/v4",
            headers={
                "Authorization": f"Bearer {token}",
                "Content-Type": "application/json",
            },
            verify=verify,
        )

        self.channels = ChannelsClient(self.session)
        self.posts = PostClient(self.session)
        self.teams = TeamClient(self.session)
        self.users = UsersClient(self.session)

    async def close(self) -> None:
        await self.session.aclose()



import pytest

from aiommost.channels.schemas import ChannelTypes
from aiommost.client import MattermostClient
from aiommost.users.schemas import User


@pytest.fixture
def create_direct_channel(client: MattermostClient, create_user):
    async def inner(user_id: str | None = None):
        if not user_id:
            user: User = await create_user()
            user_id = user.uid


        return await client.channels.create_direct(
            first=user_id,
            second=user_id,
        )

    return inner


@pytest.fixture
def create_channel(faker, client, create_team):
    async def inner(
        team_id: str | None = None,
        name: str | None = None,
        display_name: str | None = None,
        channel_type: ChannelTypes = ChannelTypes.PUBLIC,
        header: str | None = None,
        purpose: str | None = None,
    ):
        if not team_id:
            team = await create_team()
            team_id = team.uid

        return await client.channels.create(
            team_id=team_id,
            name=name or faker.pystr(min_chars=4, max_chars=20).lower(),
            display_name=display_name or faker.pystr(),
            channel_type=channel_type,
            header=header,
            purpose=purpose,
        )

    return inner



import orjson
from httpx import Response, codes


class ClientError(Exception):
    """Mattermost client error."""


class BadRequestError(ClientError):
    """Bad request error."""

    def __init__(self, code: str, message: str, detailed: str, request_id: str) -> None:
        super().__init__(f"[{request_id}] {message}")
        self.code = code
        self.request_id = request_id
        self.message = message
        self.detailed = detailed


def validate(response: Response) -> None:
    if response.status_code in {codes.BAD_REQUEST, codes.NOT_FOUND}:
        error = orjson.loads(response.content)
        raise BadRequestError(
            code=error["id"],
            request_id=error.get("request_id"),
            message=error["message"],
            detailed=error["detailed_error"],
        )

    response.raise_for_status()



from pydantic import Field

from aiommost.schemas import Schema


class User(Schema):
    uid: str = Field(alias="id")
    username: str
    email: str
    firstname: str = Field(alias="first_name")
    lastname: str = Field(alias="last_name")
    nickname: str



import pytest

from aiommost.client import MattermostClient


@pytest.fixture
def create_user(client: MattermostClient, fake):
    async def inner(username: str | None = None,
                    email: str | None = None,
                    password: str | None = None):
        return await client.users.create(
            username=username or fake.pystr().lower(),
            email=email or fake.email(),
            password=password or fake.password(),
        )

    return inner



from typing import Any

import orjson
from httpx import AsyncClient

from aiommost import errors
from aiommost.posts.schemas import Post


class PostClient:

    def __init__(self, session: AsyncClient) -> None:
        self.session = session

    async def create(
        self,
        channel_id: str,
        message: str,
        root_id: str | None = None,
        file_ids: list[str] | None = None,
        props: dict[str, Any] | None = None,
    ) -> Post:
        """Create new post in the channel."""
        url = "/posts"
        request = {
            "channel_id": channel_id,
            "message": message,
            "root_id": root_id,
            "file_ids": file_ids,
            "props": props,
        }

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        post = orjson.loads(response.content)
        return Post(**post)



from enum import Enum

from pydantic import Field

from aiommost.schemas import Schema


class TeamTypes(Enum):
    PUBLIC = "O"
    PRIVATE = "P"


class Team(Schema):
    uid: str = Field(alias="id")
    name: str
    display_name: str
    description: str
    type: TeamTypes



import os

import pytest

from aiommost.client import MattermostClient


@pytest.fixture
def client():
    host = os.environ["TEST_MM_HOST"]
    token = os.environ["TEST_MM_TOKEN"]

    return MattermostClient(host, token)


@pytest.fixture
def simple_bot_client():
    host = os.environ["TEST_MM_HOST"]
    token = os.environ["TEST_MM_SIMPLE_TOKEN"]

    return MattermostClient(host, token)



from datetime import datetime
from enum import Enum

from pydantic import Field

from aiommost.schemas import Schema


class ChannelTypes(Enum):
    DIRECT = "D"
    PRIVATE = "P"
    PUBLIC = "O"


class Channel(Schema):
    uid: str = Field(alias="id")
    team_id: str
    category: str = Field(alias="type")
    name: str
    display_name: str
    header: str
    purpose: str
    msg_count: int | None = Field(alias="total_msg_count")
    creator_id: str
    last_post_at: datetime
    type: ChannelTypes



import pytest

from aiommost import errors
from aiommost.client import MattermostClient


async def test_direct_channel_created(client: MattermostClient, create_user):
    user = await create_user()
    await client.channels.create_direct(user.uid, user.uid)


async def test__get_by_id__found_channel(client, create_channel):
    existing = await create_channel()
    channel = await client.channels.get_by_id(channel_id=existing.uid)
    assert channel.uid == existing.uid


async def test__get_by_id__not_found_channel(client, create_channel, not_existing_id):
    with pytest.raises(errors.BadRequestError):
        await client.channels.get_by_id(channel_id=not_existing_id)
